# -*- coding: utf-8 -*-

# Copyright 2007-2008 by LivingLogic AG, Bayreuth/Germany.
# Copyright 2007-2008 by Walter Dörwald
#
# All Rights Reserved
#
# See ll/__init__.py for the license


"""
Python Codec for XML decoding
"""


import codecs

from _xml_codec import detectencoding as _detectencoding, fixencoding as _fixencoding


def decode(input, errors="strict", encoding=None):
	if encoding is None:
		encoding = _detectencoding(input, True)
	if encoding == "xml":
		raise ValueError("xml not allowed as encoding name")
	(input, consumed) = codecs.getdecoder(encoding)(input, errors)
	return (_fixencoding(input, unicode(encoding), True), consumed)


def encode(input, errors="strict", encoding=None):
	consumed = len(input)
	if encoding is None:
		encoding = _detectencoding(input, True)
	else:
		input = _fixencoding(input, unicode(encoding), True)
	if encoding == "xml":
		raise ValueError("xml not allowed as encoding name")
	info = codecs.lookup(encoding)
	return (info.encode(input, errors)[0], consumed)


class IncrementalDecoder(codecs.IncrementalDecoder):
	def __init__(self, errors="strict", encoding=None):
		self.decoder = None
		self.encoding = encoding
		codecs.IncrementalDecoder.__init__(self, errors)
		self._errors = errors # Store ``errors`` somewhere else, because we have to hide it in a property
		self.buffer = ""
		self.headerfixed = False

	def iterdecode(self, input):
		for part in input:
			result = self.decode(part, False)
			if result:
				yield result
		result = self.decode("", True)
		if result:
			yield result

	def decode(self, input, final=False):
		# We're doing basically the same as a ``BufferedIncrementalDecoder``,
		# but since  the buffer is only relevant until the encoding has been detected
		# (in which case the buffer of the underlying codec might kick in),
		# we're implementing buffering ourselves to avoid some overhead.
		if self.decoder is None:
			input = self.buffer + input
			self.encoding = _detectencoding(input, final)
			if self.encoding is None:
				self.buffer = input # retry the complete input on the next call
				return u"" # no encoding determined yet, so no output
			if self.encoding == "xml":
				raise ValueError("xml not allowed as encoding name")
			self.buffer = "" # isn't needed any more, as the decoder might keep its own buffer
			self.decoder = codecs.getincrementaldecoder(self.encoding)(self._errors)
		if self.headerfixed:
			return self.decoder.decode(input, final)
		# If we haven't fixed the header yet, the content of ``self.buffer`` is a ``unicode`` object
		output = self.buffer + self.decoder.decode(input, final)
		newoutput = _fixencoding(output, unicode(self.encoding), final)
		if newoutput is None:
			self.buffer = output # retry fixing the declaration (but keep the decoded stuff)
			return u""
		self.headerfixed = True
		return newoutput

	def reset(self):
		codecs.IncrementalDecoder.reset(self)
		self.decoder = None
		self.buffer = ""
		self.headerfixed = False

	def _geterrors(self):
		return self._errors

	def _seterrors(self, errors):
		# Setting ``errors`` must be done on the real decoder too
		if self.decoder is not None:
			self.decoder.errors = errors
		self._errors = errors
	errors = property(_geterrors, _seterrors)


class IncrementalEncoder(codecs.IncrementalEncoder):
	def __init__(self, errors="strict", encoding=None):
		self.encoder = None
		self.encoding = encoding
		codecs.IncrementalEncoder.__init__(self, errors)
		self._errors = errors # Store ``errors`` somewhere else, because we have to hide it in a property
		self.buffer = u""

	def iterencode(self, input):
		for part in input:
			result = self.encode(part, False)
			if result:
				yield result
		result = self.encode(u"", True)
		if result:
			yield result

	def encode(self, input, final=False):
		if self.encoder is None:
			input = self.buffer + input
			if self.encoding is not None:
				# Replace encoding in the declaration with the specified one
				newinput = _fixencoding(input, unicode(self.encoding), final)
				if newinput is None: # declaration not complete => Retry next time
					self.buffer = input
					return ""
				input = newinput
			else:
				# Use encoding from the XML declaration
				self.encoding = _detectencoding(input, final)
			if self.encoding is not None:
				if self.encoding == "xml":
					raise ValueError("xml not allowed as encoding name")
				info = codecs.lookup(self.encoding)
				self.encoder = info.incrementalencoder(self._errors)
				self.buffer = u""
			else:
				self.buffer = input
				return ""
		return self.encoder.encode(input, final)

	def reset(self):
		codecs.IncrementalEncoder.reset(self)
		self.encoder = None
		self.buffer = u""

	def _geterrors(self):
		return self._errors

	def _seterrors(self, errors):
		# Setting ``errors ``must be done on the real encoder too
		if self.encoder is not None:
			self.encoder.errors = errors
		self._errors = errors
	errors = property(_geterrors, _seterrors)


class StreamWriter(codecs.StreamWriter):
	def __init__(self, stream, errors="strict", encoding="utf-8", header=False):
		codecs.StreamWriter.__init__(self, stream, errors)
		self.encoder = IncrementalEncoder(errors)
		self._errors = errors

	def encode(self, input, errors='strict'):
		return (self.encoder.encode(input, False), len(input))

	def _geterrors(self):
		return self._errors

	def _seterrors(self, errors):
		# Setting ``errors`` must be done on the encoder too
		if self.encoder is not None:
			self.encoder.errors = errors
		self._errors = errors
	errors = property(_geterrors, _seterrors)


class StreamReader(codecs.StreamReader):
	def __init__(self, stream, errors="strict"):
		codecs.StreamReader.__init__(self, stream, errors)
		self.decoder = IncrementalDecoder(errors)
		self._errors = errors

	def decode(self, input, errors='strict'):
		return (self.decoder.decode(input, False), len(input))

	def _geterrors(self):
		return self._errors

	def _seterrors(self, errors):
		# Setting ``errors`` must be done on the decoder too
		if self.decoder is not None:
			self.decoder.errors = errors
		self._errors = errors
	errors = property(_geterrors, _seterrors)


def search_function(name):
	if name == "xml":
		return codecs.CodecInfo(
			name="xml",
			encode=encode,
			decode=decode,
			incrementalencoder=IncrementalEncoder,
			incrementaldecoder=IncrementalDecoder,
			streamwriter=StreamWriter,
			streamreader=StreamReader,
		)


codecs.register(search_function)



# -*- coding: utf-8 -*-

## Copyright 2005-2008 by LivingLogic AG, Bayreuth/Germany
## Copyright 2005-2008 by Walter Dörwald
##
## All Rights Reserved
##
## See ll/__init__.py for the license


"""
This module is an XIST namespace for `Relax NG`__ files.

__ http://relaxng.org/
"""


from ll.xist import xsc, sims


__docformat__ = "reStructuredText"


xmlns = "http://relaxng.org/ns/structure/1.0"


class base(xsc.Element):
	"""
	"Abstract" basis class, providing common attributes.
	"""
	register = False
	
	class Attrs(xsc.Element.Attrs):
		class ns(xsc.TextAttr): pass
		# ns is inherited from the nearest ancestor, where a ns-attribute is defined, else it's ""
		class datatypeLibrary(xsc.URLAttr): pass


class anyName(base):
	"""
	Matches any name from any namespace.
	"""
	xmlns = xmlns
	# Restriction:
	# name classes of attributes can't overlap


class attribute(base):
	"""
	Specifies an XML attribute.
	"""
	xmlns = xmlns
	# Restriction:
	# after simplification: contain only text-node-relevent nodes
	# can't be duplicated
	# infinite name classes must be enclosed in oneOrMore/zeroOrMore
	# name and name class can't be both specified

	class Attrs(base.Attrs):
		class name(xsc.TextAttr): pass


class choice(base):
	"""
	nameclass: a name matches choice if, and only if, it matches at least one of
	the subname classes. pattern: it matches a node if, and only if, at least
	one of its subpatterns matches the node
	"""
	xmlns = xmlns


class data(base):
	"""
	Specifies data of a certain kind.
	"""
	xmlns = xmlns
	class Attrs(base.Attrs):
		class type(xsc.TextAttr): pass


class define(base):
	"""
	Defines a part of a grammar pattern (also a pattern), recursion possible
	only inside an element.
	"""
	xmlns = xmlns
	class Attrs(base.Attrs):
		class name(xsc.TextAttr): required = True
		class combine(xsc.TextAttr): values = (u"interleave", u"choice")


class div(base):
	"""
	Allows logical divisions, no effect on validation, annotations can be made
	here
	"""
	xmlns = xmlns
	class Attrs(base.Attrs):
		class name(xsc.TextAttr): pass


class element_(base):
	"""
	Specifies an XML element.
	"""
	xmlns = xmlns
	xmlname = "element"

	class Attrs(base.Attrs):
		class name(xsc.TextAttr): pass
		# alternative: namespace name


class empty(base):
	"""
	Specifies empty content.
	"""
	xmlns = xmlns
	model = sims.Empty()


class except_(base):
	"""
	An :class:`except_` element can remove a name class from another (this class
	has no attributes) (inside a :class:`name` element) or it is used to remove
	a set of values from a data pattern.
	"""
	xmlns = xmlns
	xmlname = "except"


class externalRef(base):
	"""
	Reference to an extern pattern stored in a file.
	"""
	xmlns = xmlns
	model = sims.Empty()
	class Attrs(base.Attrs):
		class href(xsc.URLAttr): required = True


class grammar(base):
	"""
	A :class:`grammar` element has a single :class:`start` child element, and
	zero or more :class:`define` child elements. The :class:`start` and
	:class:`define` elements contain patterns. These patterns can contain
	:class:`ref` elements that refer to patterns defined by any of the
	:class:`define` elements in that grammar element. A :class:`grammar` pattern
	is matched by matching the pattern contained in the :class:`start` element.
	"""
	xmlns = xmlns


class group(base):
	"""
	Is implied, can be also explicitly specified: the patterns have to appear
	in the specified order (except for the attributes, they are allowed to appear
	in any order in the start tag)
	"""
	xmlns = xmlns


class include(base):
	"""
	Includes an extern grammar pattern. Can contain define parts to overwrite
	that part (same name) in the extern pattern. A possible start element
	inside include overwrites the start element of the extern pattern.
	"""
	xmlns = xmlns
	class Attrs(base.Attrs):
		class href(xsc.URLAttr): required = True


class interleave(base):
	"""
	Child elements can appear in any order, if one is a group, the order must be
	kept, other direct childs can mix between.
	"""
	xmlns = xmlns


class list(base):
	"""
	Matches whitespace seperated values.
	"""
	xmlns = xmlns


class mixed(base):
	"""
	``<mixed> p </mixed>`` is short for ``<interleave> <text/> p </interleave>``
	"""
	xmlns = xmlns


class name(base):
	"""
	Defines a class with a single name.
	"""
	xmlns = xmlns
	model = sims.NoElements()


class notAllowed(base):
	"""
	Used to make extension points in patterns.
	"""
	xmlns = xmlns
	model = sims.Empty()


class nsName(base):
	"""
	Allows any name in a specific namespace.
	"""
	xmlns = xmlns
	model = sims.Elements(except_)
	class Attrs(base.Attrs):
		class ns(xsc.URLAttr): pass


class oneOrMore(base):
	"""
	There can be one or more recurrence of the enclosed pattern.
	"""
	xmlns = xmlns


class optional(base):
	"""
	The enclosed tags can be left out.
	"""
	xmlns = xmlns


class param(base):
	"""
	Specifies parameters passed to the datatype library to determine whether a
	value is valid per a datatype.
	"""
	xmlns = xmlns
	model = sims.NoElements()
	class Attrs(base.Attrs):
		class name(xsc.TextAttr): pass


class parentRef(base):
	"""
	Escapes out of the current grammar and references a definition from the
	parent of the current grammar.
	"""
	xmlns = xmlns
	model = sims.Empty()
	class Attrs(base.Attrs):
		class name(xsc.TextAttr): required = True


class ref(base):
	"""
	A :class:`ref` pattern refers to a definition from the nearest grammar
	ancestor.
	"""
	xmlns = xmlns
	model = sims.Empty()
	class Attrs(base.Attrs):
		class name(xsc.TextAttr): required = True


class start(base):
	"""
	Required start tag inside a :class:`grammar` tag.
	"""
	xmlns = xmlns
	class Attrs(base.Attrs):
		class combine(xsc.TextAttr): values = (u"combine", u"interleave")


class text(base):
	"""
	Matches arbitrary text (one or more text nodes), including empty text.
	"""
	xmlns = xmlns
	model = sims.Empty()


class value(base):
	"""
	By default, the :class:`value` pattern will consider the string in the
	pattern to match the string in the document if the two strings are the same
	after the whitespace in both strings is normalized. Whitespace normalization
	strips leading and trailing whitespace characters, and collapses sequences
	of one or more whitespace characters to a single space character. This
	corresponds to the behaviour of an XML parser for an attribute that is
	declared as other than CDATA.
	"""
	xmlns = xmlns
	model = sims.NoElements()
	class Attrs(base.Attrs):
		class type(xsc.TextAttr): pass


class zeroOrMore(base):
	"""
	There can be zero or more recurrence of the enclosed pattern.
	"""
	xmlns = xmlns


# modeltype definitions
anyName.model = sims.Elements(except_)
attribute.model = sims.ElementsOrText(choice, data, empty, externalRef, grammar, group, interleave, list, mixed, name, notAllowed, nsName, oneOrMore, optional, parentRef, ref, text, value, zeroOrMore)
choice.model = sims.Elements(anyName, attribute, choice, data, element_, empty, externalRef, grammar, group, interleave, list, mixed, name, notAllowed, nsName, oneOrMore, optional, parentRef, ref, text, value, zeroOrMore)
data.model = sims.Elements(except_, param)
define.model = sims.Elements(attribute, choice, data, element_, empty, externalRef, grammar, group, interleave, list, mixed, notAllowed, oneOrMore, optional, parentRef, ref, text, value, zeroOrMore)
div.model = sims.Elements(define, div, include, start)
element_.model = \
except_.model = sims.Elements(anyName, attribute, choice, data, element_, empty, externalRef, grammar, group, interleave, list, mixed, name, notAllowed, nsName, oneOrMore, optional, parentRef, ref, text, value, zeroOrMore)
grammar.model = sims.Elements(start, define, div, include)
group.model = sims.Elements(attribute, choice, data, element_, externalRef, grammar, group, interleave, list, mixed, notAllowed, oneOrMore, optional, parentRef, ref, text, value, zeroOrMore)
include.model = sims.Elements(start, define, div)
interleave.model = sims.Elements(attribute, choice, data, element_, externalRef, grammar, group, interleave, list, mixed, notAllowed, oneOrMore, optional, parentRef, ref, text, value, zeroOrMore)
list.model = sims.Elements(choice, data, empty, externalRef, grammar, group, notAllowed, oneOrMore, optional, parentRef, ref, value, zeroOrMore)
mixed.model = sims.Elements(attribute, choice, data, element_, empty, externalRef, grammar, group, interleave, list, mixed, notAllowed, oneOrMore, optional, parentRef, ref, text, value, zeroOrMore)
oneOrMore.model = sims.Elements(choice, data, element_, empty, externalRef, grammar, group, interleave, list, mixed, notAllowed, oneOrMore, optional, parentRef, ref, text, value, zeroOrMore)
optional.model = sims.Elements(attribute, choice, data, element_, empty, externalRef, grammar, group, interleave, list, mixed, notAllowed, oneOrMore, optional, parentRef, ref, text, value, zeroOrMore)
start.model = sims.Elements(attribute, choice, data, element_, empty, externalRef, grammar, group, interleave, list, mixed, notAllowed, oneOrMore, optional, parentRef, ref, text, value, zeroOrMore)
zeroOrMore.model = sims.Elements(choice, data, element_, empty, externalRef, grammar, group, interleave, list, mixed, notAllowed, oneOrMore, optional, parentRef, ref, text, value, zeroOrMore)



#! /usr/bin/env python
# -*- coding: utf-8 -*-

## Copyright 2004-2008 by LivingLogic AG, Bayreuth/Germany.
## Copyright 2004-2008 by Walter Dörwald
##
## All Rights Reserved
##
## See ll/__init__.py for the license


'''
It can be used for generating Oracle or SQL
server functions that return XML strings. This is done by embedding processing
instructions containing SQL code into XML files and transforming those files
with XIST.

An Oracle example that generates an HTML table containing the result of a search
for names in a ``person`` table might look like this::

	from ll.xist import xsc
	from ll.xist.ns import html, htmlspecials
	from ll import toxic

	class search(xsc.Element):
		def convert(self, converter):
			e = xsc.Frag(
				toxic.args("search varchar2"),
				toxic.vars("i integer;"),
				toxic.type("varchar2(32000);"),
				htmlspecials.plaintable(
					toxic.code("""
						i := 1;
						for row in (select name from person where name like search) loop
							"""),
							html.tr(
								html.th(toxic.expr("i"), align="right"),
								html.td(toxic.expr("xmlescape(row.name)"))
							),
							toxic.code("""
							i := i+1;
						end loop;
					""")
				)
			)
			return e.convert(converter)

	print toxic.compile(search().conv().string(encoding"us-ascii")).encode("us-ascii")

The same example for SQL Server might look like this::

	from ll.xist import xsc
	from ll.xist.ns import html, htmlspecials
	from ll import toxic

	class search(xsc.Element):
		def convert(self, converter):
			e = xsc.Frag(
				toxic.args("@search varchar(100)"),
				toxic.vars("declare @i integer;"),
				toxic.type("varchar(max)"),
				htmlspecials.plaintable(
					toxic.code("""
						set @i = 1;
				   
						declare @row_name varchar(100);
						declare person_cursor cursor for
							select name from person where name like @search

						open person_cursor

						while 1 = 1
						begin
							fetch next from person_cursor into @row_name;
							if (@@fetch_status != 0)
								break

							"""),
							html.tr(
								html.th(toxic.expr("@i"), align="right"),
								html.td(toxic.expr("schema.xmlescape(@row_name)"))
							),
							toxic.code("""
							set @i = @i+1;
						end

						close person_cursor
						deallocate person_cursor
					""")
				)
			)
			return e.convert(converter) 

	print toxic.xml2sqs(search().conv().string(encoding"us-ascii")).encode("us-ascii")


Running the Oracle script will give the following output (the indentation will
be different though)::

	(
		search varchar2
	)
	return varchar2
	as
		c_out varchar2(32000);
		i integer;
	begin
		c_out := c_out || '<table cellpadding="0" border="0" cellspacing="0">';
		i := 1;
		for row in (select name from person where name like search) loop
			c_out := c_out || '<tr><th align="right">';
			c_out := c_out || i;
			c_out := c_out || '</th><td>';
			c_out := c_out || xmlescape(row.name);
			c_out := c_out || '</td></tr>';
			i := i+1;
		end loop;
		c_out := c_out || '</table>';
		return c_out;
	end;

Instead of generating the XML from a single XIST element, it's of course also
possible to use an XML file. One that generates the same function as the
one above looks like this::

	<?args
		search varchar2
	?>
	<?vars
		i integer;
	?>
	<plaintable class="search">
		<?code
			i := 1;
			for row in (select name from person where name like search) loop
				?>
				<tr>
					<th align="right"><?expr i?></th>
					<td><?expr xmlescape(row.name)?></td>
				</tr>
				<?code
				i := i + 1;
			end loop;
		?>
	</plaintable>

When the file is saved as :file:`search.sqlxsc` then parsing the file,
transforming it and printing the function body works like this::

	from ll.xist import parsers
	from ll.xist.ns import html, htmlspecials
	from ll import toxic

	node = parsers.parsefile("search.sqlxsc")
	node = node.conv()
	print toxic.xml2ora(node.string(encoding="us-ascii")).encode("us-ascii")
'''


import cStringIO

from ll import misc
from ll.xist import xsc, publishers


__docformat__ = "reStructuredText"


__all__ = ["compile", "prettify"]


def _stringifyoracle(string, nchar=False):
	"""
	Format :var:`string` as multiple PL/SQL string constants or expressions.
	:var:`nchar` specifies if a ``NVARCHAR`` constant should be generated or a
	``VARCHAR``. This is a generator.
	"""
	current = []

	for c in string:
		if ord(c) < 32:
			if current:
				if nchar:
					yield u"N'%s'" % u"".join(current)
				else:
					yield u"'%s'" % u"".join(current)
				current = []
			yield u"chr(%d)" % ord(c)
		else:
			if c == u"'":
				c = u"''"
			current.append(c)
			if len(current) > 1000:
				if nchar:
					yield u"N'%s'" % u"".join(current)
				else:
					yield u"'%s'" % u"".join(current)
				current = []
	if current:
		if nchar:
			yield u"N'%s'" % u"".join(current)
		else:
			yield u"'%s'" % u"".join(current)


def _compile_oracle(string):
	"""
	TOXIC compile function for Oracle
	"""
	foundproc = False
	foundargs = []
	foundvars = []
	foundsql = []
	foundtype = u"clob"

	for (t, s) in misc.tokenizepi(string):
		if t is None:
			foundsql.append((-1, s))
		elif t == "code":
			foundsql.append((0, s))
		elif t == "expr":
			foundsql.append((1, s))
		elif t == "args":
			foundargs.append(s)
		elif t == "vars":
			foundvars.append(s)
		elif t == "type":
			foundtype = s
		elif t == "proc":
			foundproc = True
		else:
			# Treat unknown PIs as text
			foundsql.append((-1, u"<?%s %s?>" % (t, s)))

	result = []
	if foundargs:
		result.append(u"(\n\t%s\n)\n" % u",\n\t".join(foundargs))
	plaintype = foundtype
	if u"(" in plaintype:
		plaintype = plaintype[:plaintype.find(u"(")]
	isclob = plaintype.lower() in ("clob", "nclob")
	if not foundproc:
		result.append(u"return %s\n" % plaintype)
	result.append(u"as\n")
	if not foundproc:
		result.append(u"\tc_out %s;\n" % foundtype)
	if foundvars:
		result.append(u"\t%s\n" % u"".join(foundvars))
	nchar = foundtype.lower().startswith(u"n")
	if isclob:
		for arg in (u"clob", u"varchar2"):
			result.append(u"\tprocedure write(p_text in %s%s)\n" % (plaintype.rstrip(u"clob"), arg))
			result.append(u"\tas\n")
			result.append(u"\t\tbegin\n")
			if arg == u"clob":
				result.append(u"\t\t\tif p_text is not null and length(p_text) != 0 then\n")
				result.append(u"\t\t\t\tdbms_lob.append(c_out, p_text);\n")
			else:
				result.append(u"\t\t\tif p_text is not null then\n")
				result.append(u"\t\t\t\tdbms_lob.writeappend(c_out, length(p_text), p_text);\n")
			result.append(u"\t\tend if;\n")
			result.append(u"\tend;\n")
	result.append(u"begin\n")
	if isclob:
		result.append(u"\tdbms_lob.createtemporary(c_out, true);\n")
	for (mode, string) in foundsql:
		if mode == -1:
			for s in _stringifyoracle(string, nchar):
				if isclob:
					result.append(u"\twrite(%s);\n" % s)
				else:
					result.append(u"\tc_out := c_out || %s;\n" % s)
		elif mode == 0:
			result.append(string)
			result.append(u"\n")
		else: # mode == 1
			if isclob:
				result.append(u"\twrite(%s);\n" % string)
			else:
				result.append(u"\tc_out := c_out || %s;\n" % string)
	if not foundproc:
		result.append(u"\treturn c_out;\n")
	result.append(u"end;\n")
	return u"".join(result)


def _compile_sqlserver(string):
	"""
	TOXIC compile function for SQL Server
	"""
	foundproc = False
	foundargs = []
	foundvars = []
	foundsql = []
	foundtype = "varchar(max)"

	for (t, s) in misc.tokenizepi(string):
		if t is None:
			foundsql.append((-1, s))
		elif t == "code":
			foundsql.append((0, s))
		elif t == "expr":
			foundsql.append((1, s))
		elif t == "args":
			foundargs.append(s)
		elif t == "vars":
			foundvars.append(s)
		elif t == "type":
			foundtype = s
		elif t == "proc":
			foundproc = True
		else:
			# Treat unknown PIs as text
			foundsql.append((-1, "<?%s %s?>" % (t, s)))

	result = []
	if foundargs:
		result.append("(\n\t%s\n)\n" % u",\n\t".join(foundargs))
	if not foundproc:
		result.append("returns %s\n" % foundtype)
	result.append("as\n")
	result.append("begin\n")
	if not foundproc:
		result.append("\tdeclare @c_out %s;\n" % foundtype)
		result.append("\tset @c_out = '';\n")
	if foundvars:
		result.append("\t%s\n" % u"".join(foundvars))
	for (mode, string) in foundsql:
		if mode == -1:
			string = "'%s'" % string.replace("'", "''")
			result.append("\tset @c_out = @c_out + %s;\n" % string)
		elif mode == 0:
			result.append(string)
			result.append("\n")
		else: # mode == 1
			result.append("\tset @c_out = @c_out + set @c_out = @c_out + convert(varchar, isnull(%s, ''));\n" % string)
	if not foundproc:
		result.append("\treturn @c_out;\n")
	result.append("end;\n")
	return "".join(result)


def compile(string, mode="oracle"):
	"""
	The :class:`unicode` object :var:`string` must be a string containing
	embedded TOXIC processing instructions. :func:`compile` creates the body of
	an Oracle or SQL Server function or procedure from it. :var:`mode` can be
	either ``"oracle"`` or ``"sqlserver".
	"""
	if mode == "oracle":
		return  _compile_oracle(string)
	elif mode == "sqlserver":
		return _compile_sqlserver(string)
	raise ValueError("unknown mode %r" % mode)


def prettify(string, mode="oracle"):
	"""
	Try to fix the indentation of the PL/SQL snippet passed in. :var:`mode` can
	be either ``"oracle"`` or ``"sqlserver"``.
	"""
	lines = [line.lstrip() for line in string.splitlines()]
	newlines = []
	if mode == "oracle":
		indents = {
			"(": (0, 1),
			");": (-1, 0),
			")": (-1, 0),
			"as": (0, 1),
			"begin": (0, 1),
			"loop": (0, 1),
			"end;": (-1, 0),
			"end": (-1, 0),
			"exception": (-1, 1),
			"if": (0, 1),
			"for": (0, 1),
			"while": (0, 1),
			"elsif": (-1, 1),
			"else": (-1, 1),
		}
	elif mode == "sqlserver":
		indents = {
			"(": (0, 1),
			");": (-1, 0),
			")": (-1, 0),
			"as": (0, 1),
			"begin": (0, 1),
			"end;": (-1, 0),
			"end": (-1, 0),
		}
	else:
		raise ValueError("unknown mode %r" % mode)
	indent = 0
	firstafteras = False
	for line in lines:
		if not line:
			newlines.append("")
		else:
			prefix = line.split(None, 1)[0]
			(pre, post) = indents.get(prefix, (0, 0))
			if line.endswith("("):
				post = 1
			elif firstafteras and prefix == "begin":
				# as followed by begin has same indentation
				pre = -1
			indent = max(0, indent+pre)
			newlines.append("%s%s" % ("\t"*indent, line))
			indent = max(0, indent+post)
			if prefix == "as":
				firstafteras = True
	return "\n".join(newlines)



# -*- coding: utf-8 -*-

## Copyright 2008 by LivingLogic AG, Bayreuth/Germany
## Copyright 2008 by Walter Dörwald
##
## All Rights Reserved
##
## See ll/__init__.py for the license


from ll.xist import xsc


xmlns = "http://xmlns.livinglogic.de/xist/ns/sxtl"


###
### Processing instructions for output
###

class print_(xsc.ProcInst):
	"""
	Prints the expression in the processing instruction.
	"""
	xmlname = "print"


###
### Processing instruction for statements
###

class code(xsc.ProcInst):
	"""
	A :class:`code` processing instruction contains a statement (such as an
	assignment or augmented assignment).
	"""


###
### Processing instructions for block structures (if/elif/else and for)
###

class if_(xsc.ProcInst):
	xmlname = "if"


class elif_(xsc.ProcInst):
	xmlname = "elif"


class else_(xsc.ProcInst):
	xmlname = "else"


class end(xsc.ProcInst):
	"""
	Ends an :class:`if_` or :class:`for_`. The PI value may be the type of the
	block (either ``"if"`` or ``"for"``). If the value is empty the innermost
	block will be closed without any checks for the type of block.
	"""


class for_(xsc.ProcInst):
	xmlname = "for"



#!/usr/bin/env python
# -*- coding: utf-8 -*-

## Copyright 2005/2006 by LivingLogic AG, Bayreuth/Germany.
## Copyright 2005/2006 by Walter Dörwald
##
## All Rights Reserved
##
## See ll/__init__.py for the license


import warnings

import py.test

from ll import url


def test_escape():
	assert url.URL("%u0042").file == u"\x42"

	warnings.filterwarnings("error", category=UserWarning)
	py.test.raises(UserWarning, url.URL, "%u00")
	py.test.raises(UserWarning, url.URL, "%u00xx")

	warnings.filterwarnings("ignore", category=UserWarning)
	assert url.URL("%u00").file == u"%u00"
	assert url.URL("%u00xx").file == u"%u00xx"



# -*- coding: utf-8 -*-

## Copyright 1999-2008 by LivingLogic AG, Bayreuth/Germany
## Copyright 1999-2008 by Walter Dörwald
##
## All Rights Reserved
##
## See ll/__init__.py for the license


"""
An XIST module that contains elements that are useful for forms. These are
just abbreviations for the various ``<input type="...">`` elements.
"""


from ll.xist import xsc
from ll.xist.ns import html


__docformat__ = "reStructuredText"


xmlns = "http://xmlns.livinglogic.de/xist/ns/form"


class checkbox(html.input):
	xmlns = xmlns
	class Attrs(html.input.Attrs):
		type = None

	def convert(self, converter):
		e = html.input(self.attrs, type=u"checkbox")
		return e.convert(converter)


class edit(html.input):
	xmlns = xmlns
	class Attrs(html.input.Attrs):
		type = None

	def convert(self, converter):
		e = html.input(self.attrs, type=u"text")
		return e.convert(converter)


class radio(html.input):
	xmlns = xmlns
	class Attrs(html.input.Attrs):
		type = None

	def convert(self, converter):
		e = html.input(self.attrs, type=u"radio")
		return e.convert(converter)


class submit(html.input):
	xmlns = xmlns
	class Attrs(html.input.Attrs):
		type = None

	def convert(self, converter):
		e = html.input(self.attrs, type=u"submit")
		return e.convert(converter)


class memo(html.textarea):
	xmlns = xmlns
	class Attrs(html.textarea.Attrs):
		class value(xsc.TextAttr): pass

	def convert(self, converter):
		e = html.textarea(self[u"value"], self.attrs.withoutnames(u"value"))
		return e.convert(converter)


class hidden(html.input):
	xmlns = xmlns
	class Attrs(html.input.Attrs):
		type = None

	def __unicode__(self):
		return u""

	def convert(self, converter):
		e = html.input(self.attrs, type=u"hidden")
		return e.convert(converter)



#! /usr/bin/env/python
# -*- coding: utf-8 -*-

## Copyright 2008 by LivingLogic AG, Bayreuth/Germany
## Copyright 2008 by Walter Dörwald
##
## All Rights Reserved
##
## See ll/__init__.py for the license


import re, datetime, StringIO

import py.test

from ll import ul4c


def check(result, source, templates={}, **variables):
	# Check with template compiled from source
	t1 = ul4c.compile(source)
	assert result == t1.renders(templates, **variables)

	# Check with template loaded again via the string interface
	t2 = ul4c.loads(t1.dumps())
	assert result == t2.renders(templates, **variables)

	# Check with template loaded again via the stream interface
	stream = StringIO.StringIO()
	t1.dump(stream)
	stream.seek(0)
	t3 = ul4c.load(stream)
	assert result == t3.renders(templates, **variables)


def checkle(result, source, templates={}, **variables):
	# Check with template compiled from source
	t1 = ul4c.compile(source)
	assert result <= t1.renders(templates, **variables)

	# Check with template loaded again via the string interface
	t2 = ul4c.loads(t1.dumps())
	assert result <= t2.renders(templates, **variables)

	# Check with template loaded again via the stream interface
	stream = StringIO.StringIO()
	t1.dump(stream)
	stream.seek(0)
	t3 = ul4c.load(stream)
	assert result <= t3.renders(templates, **variables)


def checkcompileerror(msg, source):
	try:
		ul4c.compile(source)
	except Exception, exc:
		assert re.search(msg, str(exc)) is not None
	else:
		py.test.fail("Didn't raise exception")


def checkrunerror(msg, source, templates={}, **variables):
	# Check with template compiled from source
	t1 = ul4c.compile(source)
	try:
		t1.renders(templates, **variables)
	except Exception, exc:
		assert re.search(msg, "%s.%s: %s" % (exc.__class__.__module__, exc.__class__.__name__, exc)) is not None
	else:
		py.test.fail("Didn't raise exception")

	# Check with template loaded again via the string interface
	t2 = ul4c.loads(t1.dumps())
	try:
		t2.renders(templates, **variables)
	except Exception, exc:
		assert re.search(msg, "%s.%s: %s" % (exc.__class__.__module__, exc.__class__.__name__, exc)) is not None
	else:
		py.test.fail("Didn't raise exception")

	# Check with template loaded again via the stream interface
	stream = StringIO.StringIO()
	t1.dump(stream)
	stream.seek(0)
	t3 = ul4c.load(stream)
	try:
		t3.renders(templates, **variables)
	except Exception, exc:
		assert re.search(msg, "%s.%s: %s" % (exc.__class__.__module__, exc.__class__.__name__, exc)) is not None
	else:
		py.test.fail("Didn't raise exception")


def test_format():
	tmpl = ul4c.compile("<?for (i, item) in enumerate(data)?><?print (i+1).format('>3d')?>. <?print item[0].format('.<10')?> <?print item[1]?>\n<?end for?>")
	assert str(tmpl) == unicode(tmpl)


def test_text():
	check('gurk', 'gurk')
	check(u'gurk', u'gurk')
	check(u'g\xfcrk', u'g\xfcrk')


def test_none():
	check('', '<?print None?>')

	check('no', '<?if None?>yes<?else?>no<?end if?>')


def test_false():
	check('False', '<?print False?>')

	check('no', '<?if False?>yes<?else?>no<?end if?>')


def test_true():
	check('True', '<?print True?>')

	check('yes', '<?if True?>yes<?else?>no<?end if?>')


def test_int():
	check('0', '<?print 0?>')
	check('42', '<?print 42?>')
	check('-42', '<?print -42?>')
	check('255', '<?print 0xff?>')
	check('255', '<?print 0Xff?>')
	check('-255', '<?print -0xff?>')
	check('-255', '<?print -0Xff?>')
	check('63', '<?print 0o77?>')
	check('63', '<?print 0O77?>')
	check('-63', '<?print -0o77?>')
	check('-63', '<?print -0O77?>')
	check('7', '<?print 0b111?>')
	check('7', '<?print 0B111?>')
	check('-7', '<?print -0b111?>')
	check('-7', '<?print -0B111?>')

	check('no', '<?if 0?>yes<?else?>no<?end if?>')
	check('yes', '<?if 1?>yes<?else?>no<?end if?>')
	check('yes', '<?if -1?>yes<?else?>no<?end if?>')


def test_float():
	check('0.0', '<?print 0.?>')
	check('42.0', '<?print 42.?>')
	check('-42.0', '<?print -42.?>')
	check('1e+42', '<?print 1E42?>')
	check('1e+42', '<?print 1e42?>')
	check('-1e+42', '<?print -1E42?>')
	check('-1e+42', '<?print -1e42?>')

	check('no', '<?if 0.?>yes<?else?>no<?end if?>')
	check('yes', '<?if 1.?>yes<?else?>no<?end if?>')
	check('yes', '<?if -1.?>yes<?else?>no<?end if?>')


def test_string():
	check('foo', '<?print "foo"?>')
	check('\n', '<?print "\\n"?>')
	check('\r', '<?print "\\r"?>')
	check('\t', '<?print "\\t"?>')
	check('\f', '<?print "\\f"?>')
	check('\b', '<?print "\\b"?>')
	check('\a', '<?print "\\a"?>')
	check('\x1b', '<?print "\\e"?>')
	check('"', '<?print "\\""?>')
	check("'", '<?print "\\\'"?>')
	check(u'\u20ac', u'<?print "\u20ac"?>')
	check(u'\xff', u'<?print "\\xff"?>')
	check(u'\u20ac', u'''<?print "\\u20ac"?>''')
	checkcompileerror("Unterminated string", '<?print "?>')
	check("\\xxx", '<?print "\\xxx"?>')
	check("a\nb", '<?print "a\nb"?>')

	check('no', '<?if ""?>yes<?else?>no<?end if?>')
	check('yes', '<?if "foo"?>yes<?else?>no<?end if?>')


def test_date():
	check('2000-02-29T00:00:00', '<?print 2000-02-29T.isoformat()?>')
	check('2000-02-29T12:34:00', '<?print 2000-02-29T12:34.isoformat()?>')
	check('2000-02-29T12:34:56', '<?print 2000-02-29T12:34:56.isoformat()?>')
	check('2000-02-29T12:34:56.987654', '<?print 2000-02-29T12:34:56.987654.isoformat()?>')
	check('yes', '<?if 2000-02-29T12:34:56.987654?>yes<?else?>no<?end if?>')


def test_list():
	check('', '<?for item in []?><?print item?>;<?end for?>')
	check('1;', '<?for item in [1]?><?print item?>;<?end for?>')
	check('1;', '<?for item in [1,]?><?print item?>;<?end for?>')
	check('1;2;', '<?for item in [1, 2]?><?print item?>;<?end for?>')
	check('1;2;', '<?for item in [1, 2,]?><?print item?>;<?end for?>')
	check('no', '<?if []?>yes<?else?>no<?end if?>')
	check('yes', '<?if [1]?>yes<?else?>no<?end if?>')


def test_dict():
	check('', '<?for (key, value) in {}.items()?><?print key?>:<?print value?>\n<?end for?>')
	check('1:2\n', '<?for (key, value) in {1:2}.items()?><?print key?>:<?print value?>\n<?end for?>')
	check('1:2\n', '<?for (key, value) in {1:2,}.items()?><?print key?>:<?print value?>\n<?end for?>')
	# With duplicate keys, later ones simply overwrite earlier ones
	check('1:3\n', '<?for (key, value) in {1:2, 1: 3}.items()?><?print key?>:<?print value?>\n<?end for?>')
	check('no', '<?if {}?>yes<?else?>no<?end if?>')
	check('yes', '<?if {1:2}?>yes<?else?>no<?end if?>')


def test_code_storevar():
	check('42', '<?code x = 42?><?print x?>')
	check('xyzzy', '<?code x = "xyzzy"?><?print x?>')


def test_code_addvar():
	check('40', '<?code x = 17?><?code x += 23?><?print x?>')
	check('xyzzy', '<?code x = "xyz"?><?code x += "zy"?><?print x?>')


def test_code_subvar():
	check('-6', '<?code x = 17?><?code x -= 23?><?print x?>')


def test_code_mulvar():
	check('391', '<?code x = 17?><?code x *= 23?><?print x?>')
	check(17*'xyzzy', '<?code x = 17?><?code x *= "xyzzy"?><?print x?>')
	check(17*'xyzzy', '<?code x = "xyzzy"?><?code x *= 17?><?print x?>')


def test_code_floordivvar():
	check('2', '<?code x = 5?><?code x //= 2?><?print x?>')
	check('-3', '<?code x = -5?><?code x //= 2?><?print x?>')


def test_code_truedivvar():
	check('2.5', '<?code x = 5?><?code x /= 2?><?print x?>')
	check('-2.5', '<?code x = -5?><?code x /= 2?><?print x?>')


def test_code_modvar():
	check('4', '<?code x = 1729?><?code x %= 23?><?print x?>')


def test_code_delvar():
	checkrunerror('KeyError', '<?code x = 1729?><?code del x?><?print x?>')


def test_for_string():
	check('', '<?for c in data?>(<?print c?>)<?end for?>', data="")
	check('(g)(u)(r)(k)', '<?for c in data?>(<?print c?>)<?end for?>', data="gurk")


def test_for_list():
	check('', '<?for c in data?>(<?print c?>)<?end for?>', data="")
	check('(g)(u)(r)(k)', '<?for c in data?>(<?print c?>)<?end for?>', data=["g", "u", "r", "k"])


def test_for_dict():
	check('', '<?for c in data?>(<?print c?>)<?end for?>', data={})
	check('(a)(b)(c)', '<?for c in sorted(data)?>(<?print c?>)<?end for?>', data=dict(a=1, b=2, c=3))


def test_for_nested():
	check('[(1)(2)][(3)(4)]', '<?for list in data?>[<?for n in list?>(<?print n?>)<?end for?>]<?end for?>', data=[[1, 2], [3, 4]])


def test_break():
	check('1, 2, ', '<?for i in [1,2,3]?><?print i?>, <?if i==2?><?break?><?end if?><?end for?>')


def test_break_nested():
	check('1, 1, 2, 1, 2, 3, ', '<?for i in [1,2,3,4]?><?for j in [1,2,3,4]?><?print j?>, <?if j>=i?><?break?><?end if?><?end for?><?if i>=3?><?break?><?end if?><?end for?>')


def test_continue():
	check('1, 3, ', '<?for i in [1,2,3]?><?if i==2?><?continue?><?end if?><?print i?>, <?end for?>')


def test_continue_nested():
	check('1, 3, \n1, 3, \n', '<?for i in [1,2,3]?><?if i==2?><?continue?><?end if?><?for j in [1,2,3]?><?if j==2?><?continue?><?end if?><?print j?>, <?end for?>\n<?end for?>')


def test_if():
	check('42', '<?if data?><?print data?><?end if?>', data=42)


def test_else():
	check('42', '<?if data?><?print data?><?else?>no<?end if?>', data=42)
	check('no', '<?if data?><?print data?><?else?>no<?end if?>', data=0)


def test_block_errors():
	checkcompileerror("unclosed blocks", '<?for x in data?>')
	checkcompileerror("endif doesn't match any if", '<?for x in data?><?end if?>')
	checkcompileerror("not in any block", '<?end?>')
	checkcompileerror("not in any block", '<?end for?>')
	checkcompileerror("not in any block", '<?end if?>')
	checkcompileerror("else doesn't match any if", '<?else?>')
	checkcompileerror("unclosed blocks", '<?if data?>')
	checkcompileerror("unclosed blocks", '<?if data?><?else?>')
	checkcompileerror("duplicate else", '<?if data?><?else?><?else?>')
	checkcompileerror("else already seen in elif", '<?if data?><?else?><?elif data?>')
	checkcompileerror("else already seen in elif", '<?if data?><?elif data?><?elif data?><?else?><?elif data?>')


def test_empty():
	checkcompileerror("expression required", '<?print?>')
	checkcompileerror("expression required", '<?if?>')
	checkcompileerror("expression required", '<<?if x?><?elif?><?end if?>')
	checkcompileerror("loop expression required", '<?for?>')
	checkcompileerror("statement required", '<?code?>')
	checkcompileerror("render statement required", '<?render?>')


def test_add():
	check('42', '<?print 21+21?>')
	check('42', '<?code x=21?><?code y=21?><?print x+y?>')
	check('foobar', '<?code x="foo"?><?code y="bar"?><?print x+y?>')
	check('(f)(o)(o)(b)(a)(r)', '<?for i in data.foo+data.bar?>(<?print i?>)<?end for?>', data=dict(foo="foo", bar="bar"))


def test_sub():
	check('0', '<?print 21-21?>')
	check('0', '<?code x=21?><?code y=21?><?print x-y?>')



def test_mul():
	check(str(17*23), '<?print 17*23?>')
	check(str(17*23), '<?code x=17?><?code y=23?><?print x*y?>')
	check(17*"foo", '<?print 17*"foo"?>')
	check(17*"foo", '<?code x=17?><?code y="foo"?><?print x*y?>')
	check("foo"*17, '<?code x="foo"?><?code y=17?><?print x*y?>')
	check("foo"*17, '<?print "foo"*17?>')
	check("(foo)(bar)(foo)(bar)(foo)(bar)", '<?for i in 3*data?>(<?print i?>)<?end for?>', data=["foo", "bar"])


def test_truediv():
	check("0.5", '<?print 1/2?>')
	check("0.5", '<?code x=1?><?code y=2?><?print x/y?>')


def test_floordiv():
	check("0", '<?print 1//2?>')
	check("0", '<?code x=1?><?code y=2?><?print x//y?>')


def test_mod():
	check(str(42%17), '<?print 42%17?>')
	check(str(42%17), '<?code x=42?><?code y=17?><?print x%y?>')


def test_eq():
	check("False", '<?print 17==23?>')
	check("True", '<?print 17==17?>')
	check("False", '<?print x==23?>', x=17)
	check("True", '<?print x==23?>', x=23)


def test_ne():
	check("True", '<?print 17!=23?>')
	check("False", '<?print 17!=17?>')
	check("True", '<?print x!=23?>', x=17)
	check("False", '<?print x!=23?>', x=23)


def test_lt():
	check("True", '<?print 1<2?>')
	check("False", '<?print 2<2?>')
	check("False", '<?print 3<2?>')
	check("True", '<?print x<2?>', x=1)
	check("False", '<?print x<2?>', x=2)
	check("False", '<?print x<2?>', x=3)


def test_le():
	check("True", '<?print 1<=2?>')
	check("True", '<?print 2<=2?>')
	check("False", '<?print 3<=2?>')
	check("True", '<?print x<=2?>', x=1)
	check("True", '<?print x<=2?>', x=2)
	check("False", '<?print x<=2?>', x=3)


def test_gt():
	check("False", '<?print 1>2?>')
	check("False", '<?print 2>2?>')
	check("True", '<?print 3>2?>')
	check("False", '<?print x>2?>', x=1)
	check("False", '<?print x>2?>', x=2)
	check("True", '<?print x>2?>', x=3)


def test_ge():
	check("False", '<?print 1>=2?>')
	check("True", '<?print 2>=2?>')
	check("True", '<?print 3>=2?>')
	check("False", '<?print x>=2?>', x=1)
	check("True", '<?print x>=2?>', x=2)
	check("True", '<?print x>=2?>', x=3)


def test_nested():
	sc = "4"
	sv = "x"
	n = 4
	for i in xrange(8): # when using 10 compiling the variable will run out of registers
		sc = "(%s)+(%s)" % (sc, sc)
		sv = "(%s)+(%s)" % (sv, sv)
		n = n+n
	check(str(n), '<?print %s?>' % sc)
	check(str(n), '<?code x=4?><?print %s?>' % sv)


def test_precedence():
	check("14", '<?print 2+3*4?>')
	check("20", '<?print (2+3)*4?>')
	check("10", '<?print -2+-3*-4?>')
	check("14", '<?print --2+--3*--4?>')
	check("14", '<?print (-(-2))+(-((-3)*-(-4)))?>')
	check("42", '<?print 2*data.value?>', data=dict(value=21))
	check("42", '<?print data.value[0]?>', data=dict(value=[42]))
	check("42", '<?print data[0].value?>', data=[dict(value=42)])
	check("42", '<?print data[0][0][0]?>', data=[[[42]]])
	check("42", '<?print data.value.value[0]?>', data=dict(value=dict(value=[42])))
	check("42", '<?print data.value.value[0].value.value[0]?>', data=dict(value=dict(value=[dict(value=dict(value=[42]))])))


def test_bracket():
	sc = "4"
	sv = "x"
	for i in xrange(10):
		sc = "(%s)" % sc
		sv = "(%s)" % sv

	check("4", '<?print %s?>' % sc)
	check("4", '<?code x=4?><?print %s?>' % sv)


def test_function_now():
	checkrunerror("function u?'now' unknown", "<?print now(1)?>")
	checkrunerror("function u?'now' unknown", "<?print now(1, 2)?>")
	now = unicode(datetime.datetime.now())
	checkle(now, "<?print now()?>")


def test_function_vars():
	checkrunerror("function u?'vars' unknown", "<?print vars(1)?>")
	checkrunerror("function u?'vars' unknown", "<?print vars(1, 2)?>")
	check("yes", "<?if 'spam' in vars()?>yes<?else?>no<?end if?>", spam="eggs")


def test_function_xmlescape():
	checkrunerror("function u?'xmlescape' unknown", "<?print xmlescape()?>")
	checkrunerror("function u?'xmlescape' unknown", "<?print xmlescape(1, 2)?>")
	check("&lt;&gt;&amp;&#39;&quot;gurk", "<?print xmlescape(data)?>", data='<>&\'"gurk')


def test_function_csvescape():
	checkrunerror("function u?'csvescape' unknown", "<?print csvescape()?>")
	checkrunerror("function u?'csvescape' unknown", "<?print csvescape(1, 2)?>")
	check("", "<?print csvescape(data)?>", data=None)
	check("False", "<?print csvescape(data)?>", data=False)
	check("True", "<?print csvescape(data)?>", data=True)
	check("42", "<?print csvescape(data)?>", data=42)
	# no check for float
	check("abc", "<?print csvescape(data)?>", data="abc")
	check('"a,b,c"', "<?print csvescape(data)?>", data="a,b,c")
	check('"a""b""c"', "<?print csvescape(data)?>", data='a"b"c')
	check('"a\nb\nc"', "<?print csvescape(data)?>", data="a\nb\nc")


def test_function_str():
	checkrunerror("function u?'str' unknown", "<?print str()?>")
	checkrunerror("function u?'str' unknown", "<?print str(1, 2)?>")
	check("", "<?print str(data)?>", data=None)
	check("True", "<?print str(data)?>", data=True)
	check("False", "<?print str(data)?>", data=False)
	check("42", "<?print str(data)?>", data=42)
	check("4.2", "<?print str(data)?>", data=4.2)
	check("foo", "<?print str(data)?>", data="foo")


def test_function_int():
	checkrunerror("function u?'int' unknown", "<?print int()?>")
	checkrunerror("function u?'int' unknown", "<?print int(1, 2)?>")
	checkrunerror("int\\(\\) argument must be a string or a number, not 'NoneType'", "<?print int(data)?>", data=None)
	check("1", "<?print int(data)?>", data=True)
	check("0", "<?print int(data)?>", data=False)
	check("42", "<?print int(data)?>", data=42)
	check("4", "<?print int(data)?>", data=4.2)
	check("42", "<?print int(data)?>", data="42")
	checkrunerror("invalid literal for int\\(\\) with base 10: 'foo'", "<?print int(data)?>", data="foo")


def test_function_len():
	checkrunerror("function u?'len' unknown", "<?print len()?>")
	checkrunerror("function u?'len' unknown", "<?print len(1, 2)?>")
	checkrunerror("object of type 'NoneType' has no len", "<?print len(data)?>", data=None)
	checkrunerror("object of type 'bool' has no len", "<?print len(data)?>", data=True)
	checkrunerror("object of type 'bool' has no len", "<?print len(data)?>", data=False)
	checkrunerror("object of type 'int' has no len", "<?print len(data)?>", data=42)
	checkrunerror("object of type 'float' has no len", "<?print len(data)?>", data=4.2)
	check("42", "<?print len(data)?>", data=42*"?")
	check("42", "<?print len(data)?>", data=42*[None])
	check("42", "<?print len(data)?>", data=dict.fromkeys(xrange(42)))


def test_function_enumerate():
	checkrunerror("function u?'enumerate' unknown", "<?print enumerate()?>")
	checkrunerror("function u?'enumerate' unknown", "<?print enumerate(1, 2)?>")
	code = "<?for (i, value) in enumerate(data)?><?print i?>:<?print value?>\n<?end for?>"
	checkrunerror("'NoneType' object is not iterable", code, data=None)
	checkrunerror("'bool' object is not iterable", code, data=True)
	checkrunerror("'bool' object is not iterable", code, data=False)
	checkrunerror("'int' object is not iterable", code, data=42)
	checkrunerror("'float' object is not iterable", code, data=4.2)
	check("0:f\n1:o\n2:o\n", code, data="foo")
	check("0:foo\n1:bar\n", code, data=["foo", "bar"])
	check("0:foo\n", code, data=dict(foo=True))


def test_function_isnone():
	checkrunerror("function u?'isnone' unknown", "<?print isnone()?>")
	checkrunerror("function u?'isnone' unknown", "<?print isnone(1, 2)?>")
	code = "<?print isnone(data)?>"
	check("True", code, data=None)
	check("False", code, data=True)
	check("False", code, data=False)
	check("False", code, data=42)
	check("False", code, data=4.2)
	check("False", code, data="foo")
	check("False", code, data=datetime.datetime.now())
	check("False", code, data=())
	check("False", code, data=[])
	check("False", code, data={})


def test_function_isbool():
	checkrunerror("function u?'isbool' unknown", "<?print isbool()?>")
	checkrunerror("function u?'isbool' unknown", "<?print isbool(1, 2)?>")
	code = "<?print isbool(data)?>"
	check("False", code, data=None)
	check("True", code, data=True)
	check("True", code, data=False)
	check("False", code, data=42)
	check("False", code, data=4.2)
	check("False", code, data="foo")
	check("False", code, data=datetime.datetime.now())
	check("False", code, data=())
	check("False", code, data=[])
	check("False", code, data={})


def test_function_isint():
	checkrunerror("function u?'isint' unknown", "<?print isint()?>")
	checkrunerror("function u?'isint' unknown", "<?print isint(1, 2)?>")
	code = "<?print isint(data)?>"
	check("False", code, data=None)
	check("False", code, data=True)
	check("False", code, data=False)
	check("True", code, data=42)
	check("False", code, data=4.2)
	check("False", code, data="foo")
	check("False", code, data=datetime.datetime.now())
	check("False", code, data=())
	check("False", code, data=[])
	check("False", code, data={})


def test_function_isfloat():
	checkrunerror("function u?'isfloat' unknown", "<?print isfloat()?>")
	checkrunerror("function u?'isfloat' unknown", "<?print isfloat(1, 2)?>")
	code = "<?print isfloat(data)?>"
	check("False", code, data=None)
	check("False", code, data=True)
	check("False", code, data=False)
	check("False", code, data=42)
	check("True", code, data=4.2)
	check("False", code, data="foo")
	check("False", code, data=datetime.datetime.now())
	check("False", code, data=())
	check("False", code, data=[])
	check("False", code, data={})


def test_function_isstr():
	checkrunerror("function u?'isstr' unknown", "<?print isstr()?>")
	checkrunerror("function u?'isstr' unknown", "<?print isstr(1, 2)?>")
	code = "<?print isstr(data)?>"
	check("False", code, data=None)
	check("False", code, data=True)
	check("False", code, data=False)
	check("False", code, data=42)
	check("False", code, data=4.2)
	check("True", code, data="foo")
	check("False", code, data=datetime.datetime.now())
	check("False", code, data=())
	check("False", code, data=[])
	check("False", code, data={})


def test_function_isdate():
	checkrunerror("function u?'isdate' unknown", "<?print isdate()?>")
	checkrunerror("function u?'isdate' unknown", "<?print isdate(1, 2)?>")
	code = "<?print isdate(data)?>"
	check("False", code, data=None)
	check("False", code, data=True)
	check("False", code, data=False)
	check("False", code, data=42)
	check("False", code, data=4.2)
	check("False", code, data="foo")
	check("True", code, data=datetime.datetime.now())
	check("False", code, data=())
	check("False", code, data=[])
	check("False", code, data={})


def test_function_islist():
	checkrunerror("function u?'islist' unknown", "<?print islist()?>")
	checkrunerror("function u?'islist' unknown", "<?print islist(1, 2)?>")
	code = "<?print islist(data)?>"
	check("False", code, data=None)
	check("False", code, data=True)
	check("False", code, data=False)
	check("False", code, data=42)
	check("False", code, data=4.2)
	check("False", code, data="foo")
	check("False", code, data=datetime.datetime.now())
	check("True", code, data=())
	check("True", code, data=[])
	check("False", code, data={})


def test_function_isdict():
	checkrunerror("function u?'isdict' unknown", "<?print isdict()?>")
	checkrunerror("function u?'isdict' unknown", "<?print isdict(1, 2)?>")
	code = "<?print isdict(data)?>"
	check("False", code, data=None)
	check("False", code, data=True)
	check("False", code, data=False)
	check("False", code, data=42)
	check("False", code, data=4.2)
	check("False", code, data="foo")
	check("False", code, data=datetime.datetime.now())
	check("False", code, data=())
	check("False", code, data=[])
	check("True", code, data={})


def test_function_repr():
	checkrunerror("function u?'repr' unknown", "<?print repr()?>")
	checkrunerror("function u?'repr' unknown", "<?print repr(1, 2)?>")
	code = "<?print repr(data)?>"
	check("None", code, data=None)
	check("True", code, data=True)
	check("False", code, data=False)
	check("42", code, data=42)
	# no test for float
	check("'foo'", code, data="foo")
	# no test for tuples, lists and dicts


def test_function_chr():
	checkrunerror("function u?'chr' unknown", "<?print chr()?>")
	checkrunerror("function u?'chr' unknown", "<?print chr(1, 2)?>")
	code = "<?print chr(data)?>"
	check("\x00", code, data=0)
	check("a", code, data=ord("a"))
	check(u"\u20ac", code, data=0x20ac)


def test_function_ord():
	checkrunerror("function u?'ord' unknown", "<?print ord()?>")
	checkrunerror("function u?'ord' unknown", "<?print ord(1, 2)?>")
	code = "<?print ord(data)?>"
	check("0", code, data="\x00")
	check(str(ord("a")), code, data="a")
	check(str(0x20ac), code, data=u"\u20ac")


def test_function_hex():
	checkrunerror("function u?'hex' unknown", "<?print hex()?>")
	checkrunerror("function u?'hex' unknown", "<?print hex(1, 2)?>")
	code = "<?print hex(data)?>"
	check("0x0", code, data=0)
	check("0xff", code, data=0xff)
	check("0xffff", code, data=0xffff)
	check("-0xffff", code, data=-0xffff)


def test_function_oct():
	checkrunerror("function u?'oct' unknown", "<?print oct()?>")
	checkrunerror("function u?'oct' unknown", "<?print oct(1, 2)?>")
	code = "<?print oct(data)?>"
	check("0o0", code, data=0)
	check("0o77", code, data=077)
	check("0o7777", code, data=07777)
	check("-0o7777", code, data=-07777)


def test_function_bin():
	checkrunerror("function u?'bin' unknown", "<?print bin()?>")
	checkrunerror("function u?'bin' unknown", "<?print bin(1, 2)?>")
	code = "<?print bin(data)?>"
	check("0b0", code, data=0)
	check("0b11", code, data=3)
	check("-0b1111", code, data=-15)


def test_function_sorted():
	checkrunerror("function u?'sorted' unknown", "<?print sorted()?>")
	checkrunerror("function u?'sorted' unknown", "<?print sorted(1, 2)?>")
	code = "<?for i in sorted(data)?><?print i?><?end for?>"
	check("gkru", code, data="gurk")
	check("24679", code, data="92746")
	check("012", code, data={0: "zero", 1: "one", 2: "two"})


def test_function_range():
	checkrunerror("function u?'sorted' unknown", "<?print sorted()?>")
	code = "<?for i in range(data)?><?print i?><?end for?>"
	check("", code, data=-10)
	check("", code, data=0)
	check("0", code, data=1)
	check("01234", code, data=5)
	code = "<?for i in range(data[0], data[1])?><?print i?><?end for?>"
	check("", code, data=[0, -10])
	check("", code, data=[0, 0])
	check("01234", code, data=[0, 5])
	check("-5-4-3-2-101234", code, data=[-5, 5])
	code = "<?for i in range(data[0], data[1], data[2])?><?print i?><?end for?>"
	check("", code, data=[0, -10, 1])
	check("", code, data=[0, 0, 1])
	check("02468", code, data=[0, 10, 2])
	check("", code, data=[0, 10, -2])
	check("108642", code, data=[10, 0, -2])
	check("", code, data=[10, 0, 2])


def test_method_upper():
	check("GURK", "<?print 'gurk'.upper()?>")


def test_method_lower():
	check("gurk", "<?print 'GURK'.lower()?>")


def test_method_startswith():
	check("True", "<?print 'gurkhurz'.startswith('gurk')?>")
	check("False", "<?print 'gurkhurz'.startswith('hurz')?>")


def test_method_endswith():
	check("True", "<?print 'gurkhurz'.endswith('hurz')?>")
	check("False", "<?print 'gurkhurz'.endswith('gurk')?>")


def test_method_strip():
	check("gurk", r"<?print ' \t\r\ngurk \t\r\n'.strip()?>")
	check("gurk", r"<?print 'xyzzygurkxyzzy'.strip('xyz')?>")


def test_method_lstrip():
	check("gurk \t\r\n", r"<?print ' \t\r\ngurk \t\r\n'.lstrip()?>")
	check("gurkxyzzy", r"<?print 'xyzzygurkxyzzy'.lstrip('xyz')?>")


def test_method_rstrip():
	check(" \t\r\ngurk", r"<?print ' \t\r\ngurk \t\r\n'.rstrip()?>")
	check("xyzzygurk", r"<?print 'xyzzygurkxyzzy'.rstrip('xyz')?>")


def test_method_split():
	check("(g)(u)(r)(k)", r"<?for item in ' \t\r\ng \t\r\nu \t\r\nr \t\r\nk \t\r\n'.split()?>(<?print item?>)<?end for?>")
	check("(g)(u \t\r\nr \t\r\nk \t\r\n)", r"<?for item in ' \t\r\ng \t\r\nu \t\r\nr \t\r\nk \t\r\n'.split(None, 1)?>(<?print item?>)<?end for?>")
	check("()(g)(u)(r)(k)()", r"<?for item in 'xxgxxuxxrxxkxx'.split('xx')?>(<?print item?>)<?end for?>")
	check("()(g)(uxxrxxkxx)", r"<?for item in 'xxgxxuxxrxxkxx'.split('xx', 2)?>(<?print item?>)<?end for?>")


def test_method_rsplit():
	check("(g)(u)(r)(k)", r"<?for item in ' \t\r\ng \t\r\nu \t\r\nr \t\r\nk \t\r\n'.rsplit()?>(<?print item?>)<?end for?>")
	check("( \t\r\ng \t\r\nu \t\r\nr)(k)", r"<?for item in ' \t\r\ng \t\r\nu \t\r\nr \t\r\nk \t\r\n'.rsplit(None, 1)?>(<?print item?>)<?end for?>")
	check("()(g)(u)(r)(k)()", r"<?for item in 'xxgxxuxxrxxkxx'.rsplit('xx')?>(<?print item?>)<?end for?>")
	check("(xxgxxuxxr)(k)()", r"<?for item in 'xxgxxuxxrxxkxx'.rsplit('xx', 2)?>(<?print item?>)<?end for?>")


def test_method_replace():
	check('goork', r"<?print 'gurk'.replace('u', 'oo')?>")


def test_method_format():
	now = datetime.datetime.now()
	format = "%Y-%m-%d %H:%M:%S"
	check(now.strftime(format), r"<?print data.format('%s')?>" % format, data=now)


def test_method_isoformat():
	now = datetime.datetime.now()
	check(now.isoformat(), r"<?print data.isoformat()?>", data=now)


def test_render():
	t = ul4c.compile('<?print prefix?><?print data?><?print suffix?>')
	check('(f)(o)(o)', '<?for c in data?><?render t(data=c, prefix="(", suffix=")")?><?end for?>', dict(t=t), data='foo')


def test_render_var():
	t = ul4c.compile('<?code x += 1?><?print x?>')
	check('42,43,42', '<?print x?>,<?render t(x=x)?>,<?print x?>', dict(t=t), x=42)


def test_parse():
	check('42', '<?print data.Noner?>', data=dict(Noner=42))



#! /usr/bin/env/python
# -*- coding: utf-8 -*-

## Copyright 2005/2006 by LivingLogic AG, Bayreuth/Germany.
## Copyright 2005/2006 by Walter Dörwald
##
## All Rights Reserved
##
## See ll/__init__.py for the license


import sys, re

from ll import misc


# The following includes \x00 in addition to those characters defined in
# http://www.w3.org/TR/2004/REC-xml11-20040204/#NT-RestrictedChar
restrictedchars = re.compile(u"[\x00-\x08\x0B-\x0C\x0E-\x1F\x7F-\x84\x86-\x9F]")


escape_input = u"".join([unichr(i) for i in xrange(1000)] + [unichr(i) for i in xrange(sys.maxunicode-10, sys.maxunicode+1)])


def test_xmlescape():
	escape_output = []
	for c in escape_input:
		if c==u"&":
			escape_output.append(u"&amp;")
		elif c==u"<":
			escape_output.append(u"&lt;")
		elif c==u">":
			escape_output.append(u"&gt;")
		elif c==u'"':
			escape_output.append(u"&quot;")
		elif c==u"'":
			escape_output.append(u"&apos;")
		elif restrictedchars.match(c) is not None:
			escape_output.append(u"&#%d;" % ord(c))
		else:
			escape_output.append(c)
	escape_output = "".join(escape_output)
	assert misc.xmlescape(escape_input) == escape_output


def test_xmlescape_text():
	escape_output = []
	for c in escape_input:
		if c==u"&":
			escape_output.append(u"&amp;")
		elif c==u"<":
			escape_output.append(u"&lt;")
		elif c==u">":
			escape_output.append(u"&gt;")
		elif restrictedchars.match(c) is not None:
			escape_output.append(u"&#%d;" % ord(c))
		else:
			escape_output.append(c)
	escape_output = "".join(escape_output)
	assert misc.xmlescape_text(escape_input) == escape_output




def test_xmlescape():
	escape_output = []
	for c in escape_input:
		if c==u"&":
			escape_output.append(u"&amp;")
		elif c==u"<":
			escape_output.append(u"&lt;")
		elif c==u">":
			escape_output.append(u"&gt;")
		elif c==u'"':
			escape_output.append(u"&quot;")
		elif restrictedchars.match(c) is not None:
			escape_output.append(u"&#%d;" % ord(c))
		else:
			escape_output.append(c)
	escape_output = "".join(escape_output)
	assert misc.xmlescape_attr(escape_input) == escape_output



# -*- coding: utf-8 -*-

## Copyright 1999-2008 by LivingLogic AG, Bayreuth/Germany
## Copyright 1999-2008 by Walter Dörwald
##
## All Rights Reserved
##
## Permission is hereby granted, free of charge, to any person obtaining a copy
## of this software and associated documentation files (the "Software"), to deal
## in the Software without restriction, including without limitation the rights
## to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
## copies of the Software, and to permit persons to whom the Software is
## furnished to do so, subject to the following conditions:
##
## The above copyright notice and this permission notice shall be included in
## all copies or substantial portions of the Software.
##
## THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
## IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
## FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
## AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
## LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
## OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
## THE SOFTWARE.
##
##
## This software includes spark by John Aycock
## http://pages.cpsc.ucalgary.ca/~aycock/spark/
##
##  Copyright (c) 1998-2002 John Aycock
##  
##  Permission is hereby granted, free of charge, to any person obtaining
##  a copy of this software and associated documentation files (the
##  "Software"), to deal in the Software without restriction, including
##  without limitation the rights to use, copy, modify, merge, publish,
##  distribute, sublicense, and/or sell copies of the Software, and to
##  permit persons to whom the Software is furnished to do so, subject to
##  the following conditions:
##  
##  The above copyright notice and this permission notice shall be
##  included in all copies or substantial portions of the Software.
##  
##  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
##  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
##  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
##  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
##  CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
##  TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
##  SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
##
##
## This software includes sgmlop by Fredrik Lundh
## http://effbot.org/zone/sgmlop-index.htm
##
## Copyright (c) 1998-2007 by Secret Labs AB
## Copyright (c) 1998-2007 by Fredrik Lundh
##
## fredrik@pythonware.com
## http://www.pythonware.com
##
## By obtaining, using, and/or copying this software and/or its
## associated documentation, you agree that you have read, understood,
## and will comply with the following terms and conditions:
##
## Permission to use, copy, modify, and distribute this software and its
## associated documentation for any purpose and without fee is hereby
## granted, provided that the above copyright notice appears in all
## copies, and that both that copyright notice and this permission notice
## appear in supporting documentation, and that the name of Secret Labs
## AB or the author not be used in advertising or publicity pertaining to
## distribution of the software without specific, written prior
## permission.
##
## SECRET LABS AB AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO
## THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
## FITNESS.  IN NO EVENT SHALL SECRET LABS AB OR THE AUTHOR BE LIABLE FOR
## ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
## WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
## ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
## OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

try:
	__import__("pkg_resources").declare_namespace(__name__)
except Exception:
	pass



# Python string formatting

from math import log
try:
    import locale
except:
    locale = None
try:
    import fpformat
except:
    fpformat = None

# Except for errors in the format string.
class FormatError(StandardError):
    pass

strict_format_errors = False

class ConversionTypes:
    Binary      = 'b'   # Base-2
    Character   = 'c'   # Print as character
    Decimal     = 'd'   # Decimal integer
    Exponent    = 'e'   # Exponential notation
    ExponentUC  = 'E'   # Exponential notation with upper case 'E'
    Fixed       = 'f'   # Fixed-point
    FixedUC     = 'F'   # Fixed-point with upper case
    General     = 'g'   # General number notation
    GeneralUC   = 'G'   # General number notation with upper case 'E'
    Number      = 'n'   # Number in locale-specific format
    Octal       = 'o'   # Octal
    Repr        = 'r'   # In repr() format
    String      = 's'   # Convert using str()
    Hex         = 'x'   # Base 16
    HexUC       = 'X'   # Base 16 upper case
    Percentage  = '%'   # As percentage

ConversionTypes.All = set(ConversionTypes.__dict__.values())

# Parse the standard conversion spec. Note that I don't use
# regex here because I'm trying to eliminate external dependencies
# as much as possible.
def parse_std_conversion(spec):
    length = None
    precision = None
    ctype = None
    align = None
    fill_char = None
    sign = None

    index = 0
    spec_len = len(spec)

    # If the second char is an alignment token,
    # then parse the fill char
    if spec_len >=2 and spec[ 1 ] in '<>=^':
        fill_char = spec[ 0 ]
        align = spec[ 1 ]
        index = 2
    # Otherwise, parse the alignment token
    elif spec_len >= 1 and spec[ 0 ] in '<>=^':
        align = spec[ 0 ]
        index = 1

    # Parse the various sign options
    if index < spec_len and spec[ index ] in ' +-(':
        sign = spec[ index ]
        index += 1
        if index < spec_len and spec[ index ] == ')':
            index += 1

    # The special case for 0-padding (backwards compat)
    if fill_char == None and index < spec_len and spec[ index ] == '0':
        fill_char = '0'
        if align == None:
            align = '='
        index += 1

    # Parse field width
    saveindex = index
    while index < spec_len and spec[index].isdigit():
        index += 1

    if index > saveindex:
        length = int(spec[saveindex : index])

    # Parse field precision
    if index < spec_len and spec[index] == '.':
        index += 1
        saveindex = index
        while index < spec_len and spec[index].isdigit():
            index += 1
        if index > saveindex:
            precision = int(spec[saveindex:index])

    # Finally, parse the type field
    remaining = spec_len - index
    if remaining > 1:
        return None     # Invalid conversion spec

    if remaining == 1:
        ctype = spec[index]
        if ctype not in ConversionTypes.All:
            return None

    return (fill_char, align, sign, length, precision, ctype)

# Convert to int, and split into sign part and magnitude part
def to_int(val):
    val = int(val)
    if val < 0: return '-', -val
    return '+', val

# Convert to float, and split into sign part and magnitude part
def to_float(val):
    val = float(val)
    if val < 0: return '-', -val
    return '+', val

# Pure python implementation of the C printf 'e' format specificer
def sci(val,precision,letter='e'):
    # Split into sign and magnitude (not really needed for formatting
    # since we already did this part. Mainly here in case 'sci'
    # ever gets split out as an independent function.)
    sign = ''
    if val < 0:
        sign = '-'
        val = -val

    # Calculate the exponent
    exp = int(floor(log(val,10)))

    # Normalize the value
    val *= 10**-exp

    # If the value is exactly an integer, then we don't want to
    # print *any* decimal digits, regardless of precision
    if val == floor(val):
        val = int(val)
    else:
        # Otherwise, round it based on precision
        val = round(val,precision)
        # The rounding operation might have increased the
        # number to where it is no longer normalized, if so
        # then adjust the exponent.
        if val >= 10.0:
            exp += 1
            val = val * 0.1

    # Convert the exponent to a string using only str().
    # The existing C printf always prints at least 2 digits.
    esign = '+'
    if exp < 0:
        exp = -exp
        esign = '-'
    if exp < 10: exp = '0' + str(exp)
    else: exp = str(exp)

    # The final result
    return sign + str(val) + letter + esign + exp

# The standard formatter
def format_builtin_type(value, spec):

    # Parse the conversion spec
    conversion = parse_std_conversion(spec)
    if conversion is None:
        raise FormatError("Invalid conversion spec: " + spec)

    # Unpack the conversion spec
    fill_char, align, sign_char, length, precision, ctype = conversion

    # Choose a default conversion type
    if ctype == None:
        if isinstance(value, int) or isinstance(value, long):
            ctype = ConversionTypes.Decimal
        elif isinstance(value, float):
            ctype = ConversionTypes.General
        else:
            ctype = ConversionTypes.String

    sign = None

    # Conversion types that resolve to other types
    if ctype == ConversionTypes.Percentage:
        ctype = ConversionTypes.Fixed
        value = float(value) * 100.0

    if ctype == ConversionTypes.Binary:
        result = ''
        sign, value = to_int(value)
        while value:
            if value & 1: result = '1' + result
            else: result = '0' + result
            value >>= 1
        if len(result) == 0:
            result = '0'
    elif ctype == ConversionTypes.Octal:
        sign, value = to_int(value)
        result = oct(value)
    elif ctype == ConversionTypes.Hex:
        sign, value = to_int(value)
        result = hex(value)
    elif ctype == ConversionTypes.HexUC:
        sign, value = to_int(value)
        result = hex(value).upper()
    elif ctype == ConversionTypes.Character:
        result = chr(int( value) )
    elif ctype == ConversionTypes.Decimal:
        sign, value = to_int(value)
        result = str(value)
    elif ctype == ConversionTypes.Fixed or ctype == ConversionTypes.FixedUC:
        sign, value = to_float(value)
        if fpformat and precision is not None:
            result = fpformat.fix(value, precision)
        else:
            result = str(value)
    elif ctype == ConversionTypes.General or ctype == ConversionTypes.GeneralUC:
        #Same as "e" if exponent is less than -4 or greater than precision, "f" otherwise.
        sign, value = to_float(value)
        if fpformat and precision is not None:
            if value < 0.0001 or value > 10**precision:
                result = fpformat.sci(value, precision)
            else:
                result = fpformat.fix(value, precision)
            if ctype == ConversionTypes.GeneralUC:
                result = result.upper()
        else:
            result = str(value)
    elif ctype == ConversionTypes.Exponent or ctype == ConversionTypes.ExponentUC:
        sign, value = to_float(value)
        if precision is None: precision = 5 # Duh, I dunno
        result = sci(value, precision, ctype)
    elif ctype == ConversionTypes.Number:
        sign, value = to_float(value)
        if locale:
            # For some reason, this is not working the way I would
            # expect
            result = locale.format("%f", float( value) )
        else:
            result = str(value)
    elif ctype == ConversionTypes.String:
        result = str(value)
    elif ctype == ConversionTypes.Repr:
        result = repr(value)

    # Handle the sign logic
    prefix = ''
    suffix = ''
    if sign == '-':
        if sign_char == '(': prefix, suffix = '(', ')'
        else: prefix = '-'
    elif sign == '+':
        if sign_char == '+': prefix = '+'
        elif sign_char == ' ': prefix = ' '

    # Handle the padding logic
    if length is not None:
        padding = length - len(result) - len(prefix) - len(suffix)
        if fill_char is None:
            fill_char = ' '
        if padding > 0:
            if align == '>' or align == '^':
                return fill_char * padding + prefix + result + suffix
            elif align == '=':
                return prefix + fill_char * padding + result + suffix
            else:
                return prefix + result + suffix + fill_char * padding

    return prefix + result + suffix

def cformat(template, format_hook, args, kwargs):
    # Using array types since we're going to be growing
    # a lot.
    from array import array
    array_type = 'c'

    # Use unicode array if the original string is unicode.
    if isinstance(template, unicode): array_type = 'u'
    buffer = array(array_type)

    # Track which arguments actuallly got used
    unused_args = set(kwargs.keys())
    unused_args.update(range(0, len(args)))

    # Inner function to format a field from a value and
    # conversion spec. Most details missing.
    def format_field(value, cspec):

        # See if there's a hook
        if format_hook:
            v = format_hook(value, cspec)
            if v is not None:
                return str(v)

        # See if there's a __format__ method
        elif hasattr(value, '__format__'):
            return value.__format__(cspec)

        # Default formatting
        return format_builtin_type(value, cspec)

    # Parse a field specification. Returns True if it was a valid
    # field, False if it was merely an escaped brace. (We do it
    # this way to avoid lookahead.)
    def parse_field(buffer):

        # A separate array for the field spec.
        fieldspec = array(array_type)

        # Consume from the template iterator.
        for index, ch in template_iter:
            # A sub-field. We just interpret it like a normal field,
            # and append to the fieldspec.
            if ch == '{':
                # If the very first character is an open brace, then
                # assume its an escaped (doubled) brace.
                if len(fieldspec) == 0:
                    return False

                # Here's where we catch that doubled brace
                if not parse_field(fieldspec):
                    buffer.extend('{')
                    return True

            # End of field. Now interpret it.
            elif ch == '}':
                # Convert the array to string or uni
                if array_type == 'u':
                  fieldspec = fieldspec.tosunicode()
                else:
                  fieldspec = fieldspec.tostring()

                # Check for conversion spec
                name = fieldspec
                conversion = ''
                parts = fieldspec.split(':', 1)
                if len(parts) > 1:
                    name, conversion = parts

                try:
                    first_time = True
                    # Split the field name into subfields
                    for namepart in name.split('.'):
                        # Split that part by open bracket chars
                        keyparts = namepart.split('[')
                        # The first part is just a bare name
                        key = keyparts[0]

                        # Empty strings are not allowed as field names
                        if key == '':
                            raise FormatError("empty field name at char " + str(index))

                        # The first name in the sequence is used to index
                        # the args/kwargs arrays. Subsequent names are used
                        # on the result of the previous operation.
                        if first_time:
                            first_time = False

                            # Attempt to coerce key to integer
                            try:
                                key = int(key)
                                value = args[key]
                            except ValueError:
                                # Keyword args are strings, not uni (so far)
                                value = kwargs[key]

                            # If we got no exception, then remove from
                            # unused args
                            unused_args.remove(key)
                        else:
                            # This is not the first time, so get
                            # an attribute
                            value = getattr(value, key)

                        # Now process any bracket expressions which followed
                        # the first part.
                        for key in keyparts[1:]:
                            endbracket = key.find(']')
                            if endbracket < 0 or endbracket != len(key) - 1:
                                raise FormatError("Invalid field syntax at position " + str(index))

                            # Strip off the closing bracket and try to coerce to int
                            key = key[:-1]
                            try:
                                key = int(key)
                            except ValueError:
                                pass

                            # Get the attribute
                            value = value[key]

                except (AttributeError,KeyError,IndexError), e:
                    if strict_format_errors: raise
                    buffer.extend('?' + e.__class__.__name__ + '?')
                    return True

                buffer.extend(format_field(value, conversion))
                return True
            else:
                fieldspec.append(ch)

        raise FormatError("unmatched open brace at position " + str(index))

    # Construct an iterator from the template
    template_iter = enumerate(template)
    prev = None
    for index, ch in template_iter:
        if prev == '}':
            if ch != '}':
                raise FormatError("unmatched close brace")
            else:
                buffer.append('}')
                prev = None
                continue

        if ch == '{':
            # It's a field
            if not parse_field(buffer):
                buffer.extend('{')
        elif ch != '}':
            buffer.append(ch)
        prev = ch

    if prev == '}':
        raise FormatError("unmatched close brace")

    # Complain about unused args
    if unused_args and strict_format_errors:
        raise FormatError(
            "Unused arguments: "
            + ",".join(str(x) for x in unused_args))

    # Convert the array to its proper type
    if isinstance(template, unicode):
        return buffer.tounicode()
    else:
        return buffer.tostring()

def format(template, *args, **kwargs):
    return cformat(template, None, args, kwargs)



# -*- coding: utf-8 -*-

## Copyright 2004-2008 by LivingLogic AG, Bayreuth/Germany.
## Copyright 2004-2008 by Walter Dörwald
##
## All Rights Reserved
##
## See ll/__init__.py for the license


"""
:mod:`ll.color` provides classes and functions for handling RGB colors.
"""


import colorsys

from ll import misc


__docformat__ = "reStructuredText"


class Color(object):
	"""
	A :class:`Color` object represents a color with red, green and blue
	components.
	"""
	__slots__ = ("_channels", )
	
	def __init__(self, r=0.0, g=0.0, b=0.0):
		"""
		Create a :class:`Color` with the red, green, and blue components
		:var:`r`, :var:`g` and :var:`b`. Values will be clipped to the range
		[0; 1].
		"""
		self._channels = (max(0., min(float(r), 1.0)), max(0., min(float(g), 1.0)), max(0., min(float(b), 1.0)))

	@classmethod
	def fromrgb(cls, r, g, b):
		"""
		Create a :class:`Color` object from the RGB values :var:`r`, :var:`g` and
		:var:`b`. See the :prop:`rgb` property for more info.
		"""
		c = cls()
		c.rgb = (r, g, b)
		return c

	@classmethod
	def fromrgb4(cls, r4, g4, b4):
		"""
		Create a :class:`Color` object from the 4 bit RGB values :var:`r4`,
		:var:`g4` and :var:`b4`. See the :prop:`rgb4` property for more info.
		"""
		c = cls()
		c.rgb4 = (r4, g4, b4)
		return c

	@classmethod
	def fromrgb8(cls, r8, g8, b8):
		"""
		Create a :class:`Color` object from the 8 bit RGB values :var:`r8`,
		:var:`g8` and :var:`b8`. See the :prop:`rgb8` property for more info.
		"""
		c = cls()
		c.rgb8 = (r8, g8, b8)
		return c

	@classmethod
	def fromint4(cls, int4):
		"""
		Create a :class:`Color` object from the 12 bit RGB integer :var:`int4`.
		See the :prop:`int4` property for more info.
		"""
		c = cls()
		c.int4 = int4
		return c

	@classmethod
	def fromint8(cls, int8):
		"""
		Create a :class:`Color` object from the 24 bit RGB integer :var:`int8`.
		See the :prop:`int8` property for more info.
		"""
		c = cls()
		c.int8 = int8
		return c

	@classmethod
	def fromcss(cls, s):
		"""
		Create a :class:`Color` object from the CSS color string :var:`s`.
		See the :prop:`css` property for more info.
		"""
		c = cls()
		c.css = s
		return c

	@classmethod
	def fromhsv(cls, h, s, v):
		"""
		Create a :class:`Color` object from the hue, saturation and value values
		:var:`h`, :var:`s` and :var:`v`. See the :prop:`hsv` property for more
		info.
		"""
		c = cls()
		c.hsv = (h, s, v)
		return c

	@classmethod
	def fromhls(cls, h, l, s):
		"""
		Create a :class:`Color` object from the hue, luminance and saturation
		values :var:`h`, :var:`l` and :var:`s`. See the :prop:`hls` property
		for more info.
		"""
		c = cls()
		c.hls = (h, l, s)
		return c

	def __repr__(self):
		return "Color(%r, %r, %r)" % tuple(self._channels)

	def __str__(self):
		return self.css

	class r(misc.propclass):
		"""
		The red value as a float between 0.0 and 1.0. The value will be clipped
		on setting.
		"""
		def __get__(self):
			return self._channels[0]
		def __set__(self, r):
			self._channels = (max(0., min(float(r), 1.0)), self._channels[1], self._channels[2])

	class g(misc.propclass):
		"""
		The green value as a float between 0.0 and 1.0. The value will be clipped
		on setting.
		"""
		def __get__(self):
			return self._channels[1]
		def __set__(self, g):
			self._channels = (self._channels[0], max(0., min(float(g), 1.0)), self._channels[2])

	class b(misc.propclass):
		"""
		The blue value as a float between 0.0 and 1.0. The value will be clipped
		on setting.
		"""
		def __get__(self):
			return self._channels[2]
		def __set__(self, b):
			self._channels = (self._channels[0], self._channels[1], max(0., min(float(b), 1.0)))

	class r4(misc.propclass):
		"""
		The red value as an int between 0 and 15. The value will be clipped
		on setting.
		"""
		def __get__(self):
			return int(15.*self._channels[0]+0.5)
		def __set__(self, r4):
			self._channels = (max(0., min(r4/15., 1.)), self._channels[1], self._channels[2])

	class g4(misc.propclass):
		"""
		The green value as an int between 0 and 15. The value will be clipped
		on setting.
		"""
		def __get__(self):
			return int(15.*self._channels[1]+0.5)
		def __set__(self, g4):
			self._channels = (self._channels[0], max(0., min(g4/15., 1.)), self._channels[2])

	class b4(misc.propclass):
		"""
		The blue value as an int between 0 and 15. The value will be clipped on
		setting.
		"""
		def __get__(self):
			return int(15.*self._channels[2]+0.5)
		def __set__(self, b4):
			self._channels = (self._channels[0], self._channels[1], max(0., min(b4/15., 1.)))

	class r8(misc.propclass):
		"""
		The red value as an int between 0 and 255. The value will be clipped on
		setting.
		"""
		def __get__(self):
			return int(255.*self._channels[0]+0.5)
		def __set__(self, r8):
			self._channels = (max(0., min(r8/255., 1.)), self._channels[1], self._channels[2])

	class g8(misc.propclass):
		"""
		The green value as an int between 0 and 255. The value will be clipped on
		setting.
		"""
		def __get__(self):
			return int(255.*self._channels[1]+0.5)
		def __set__(self, g8):
			self._channels = (self._channels[0], max(0., min(g8/255., 1.)), self._channels[2])

	class b8(misc.propclass):
		"""
		The blue value as an int between 0 and 255. The value will be clipped on
		setting.
		"""
		def __get__(self):
			return int(255.*self._channels[2]+0.5)
		def __set__(self, b8):
			self._channels = (self._channels[0], self._channels[1], max(0., min(b8/255., 1.)))

	class int4(misc.propclass):
		"""
		The RGB value as a 12 bit integer. Red is bit 8-11, green is bit 4-7 and
		blue is bit 0-3)
		"""
		def __get__(self):
			rgb4 = self.rgb4
			return (rgb4[0]<<8) + (rgb4[1]<<4) + rgb4[2]
		def __set__(self, rgb4):
			self.rgb4 = (rgb4>>8 & 0xf, rgb4>>4 & 0xf, rgb4&0xf)

	class int8(misc.propclass):
		"""
		The RGB value as a 24 bit integer. Red is bit 16-23, green is bit 8-15
		and blue is bit 0-7)
		"""
		def __get__(self):
			rgb8 = self.rgb8
			return (rgb8[0]<<16) + (rgb8[1]<<8) + rgb8[2]
		def __set__(self, rgb8):
			self.rgb8 = (rgb8>>16 & 0xff, rgb8>>8 & 0xff, rgb8&0xff)

	def __getitem__(self, index):
		"""
		Return the red, green or blue value (depending on whether :var:`index` is
		0, 1 or 2) as a float value between 0.0 and 1.0.
		"""
		return self._channels[index]

	def __setitem__(self, index, value):
		"""
		Set the red, green or blue value (depending on whether :var:`index` is
		0, 1 or 2). The value should be between 0.0 and 1.0. Values outside this
		range will be clipped.
		"""
		channels = list(self._channels)
		channels[index] = max(0., min(float(value), 1.))
		self._channels = tuple(channels)

	def __len__(self):
		"""
		Always returns 3.
		"""
		return 3

	class rgb(misc.propclass):
		"""
		The red, green and blue value as a float tuple with values between 0.0
		and 1.0. Values will be clipped on setting.
		"""
		def __get__(self):
			return self._channels
		def __set__(self, rgb):
			self._channels = tuple([max(0., min(float(x), 1.)) for x in rgb])

	class rgb4(misc.propclass):
		"""
		The red, green and blue value as an int tuple with values between 0 and
		15. Values will be clipped on setting.
		"""
		def __get__(self):
			return tuple([int(15.*x+0.5) for x in self._channels])
		def __set__(self, rgb4):
			self._channels = tuple([max(0., min(x/15., 1.)) for x in rgb4])

	class rgb8(misc.propclass):
		"""
		The red, green and blue value as an int tuple with values between 0 and
		255. Values will be clipped on setting.
		"""
		def __get__(self):
			return tuple([int(255.*x+0.5) for x in self._channels])
		def __set__(self, rgb8):
			self._channels = tuple([max(0., min(x/255., 1.)) for x in rgb8])

	class css(misc.propclass):
		"""
		:var:`self` formatted as a &css; color string. On setting all formats
		from CSS2 are supported (i.e. ``'#xxx'``, ``'#xxxxxx'``, ``rgb(n, n, n)``,
		``rgb(n%, n%, n%)`` and color names like ``'red'``).
		"""
		def __get__(self):
			s = "#%06x" % self.int8
			if s[1]==s[2] and s[3]==s[4] and s[5]==s[6]:
				s = "#%s%s%s" % (s[1], s[3], s[5])
			return s
		def __set__(self, s):
			if s.startswith("#"):
				i = int(s[1:], 16)
				if len(s) == 4:
					self.int4 = i
					return
				elif len(s) == 7:
					self.int8 = i
					return
			elif s.startswith("rgb(") and s.endswith(")"):
				channels = []
				for x in s[4:-1].split(","):
					x = x.strip()
					if x.endswith("%"):
						v = float(x[:-1])/100.
					else:
						v = float(x)/255.
					channels.append(v)
				self.rgb = channels
				return
			elif s in csscolors:
				self.rgb = csscolors[s].rgb
				return
			raise ValueError("can't interpret %s as css value" % s)

	class hsv(misc.propclass):
		"""
		:var:`self` as a HSV ("hue, saturation, value") triple. All three values
		are between 0.0 and 1.0. On setting hue will be used modulo 1.0,
		saturation and value will be clipped to the range [0; 1].
		"""
		def __get__(self):
			return colorsys.rgb_to_hsv(*self.rgb)
		def __set__(self, (h, s, v)):
			self.rgb = colorsys.hsv_to_rgb(h % 1.0, max(0., min(s, 1.)), max(0., min(v, 1.)))

	class hls(misc.propclass):
		"""
		:var:`self` as a HLS ("hue, luminance, saturation") triple. All three
		values are between 0.0 and 1.0. On setting hue will be used modulo 1.0,
		luminance and saturation will be clipped to the range [0; 1].
		"""
		def __get__(self):
			return colorsys.rgb_to_hls(*self.rgb)
		def __set__(self, (h, l, s)):
			self.rgb = colorsys.hls_to_rgb(h % 1.0, max(0., min(l, 1.)), max(0., min(s, 1.)))

	class lum(misc.propclass):
		"""
		The luminance value from the :prop:`hls` property.
		"""
		def __get__(self):
			return colorsys.rgb_to_hls(*self.rgb)[1]
		def __set__(self, lum):
			(h, l, s) = self.hls
			self.rgb = colorsys.hls_to_rgb(h, max(0., min(lum, 1.)), s)

	def abslum(self, f):
		"""
		Return a copy of :var:`self` with :var:`f` added to the luminocity.
		"""
		(h, l, s) = self.hls
		return self.fromhls(h, l+f, s)

	def rellum(self, f):
		"""
		Return a copy of :var:`self` where the luminocity has been modified:
		If :var:`f` if positive the luminocity will be increased, with
		``f==1`` giving a luminocity of 1. If :var:`f` is negative, the
		luminocity will be decreased with ``:var:`f`==-1`` giving a luminocity
		of 0. ``f==0`` will leave the luminocity unchanged.
		"""
		(h, l, s) = self.hls
		if f > 0:
			l += (1-l)*f
		elif f < 0:
			l += l*f
		return self.fromhls(h, l, s)


###
### Color constants (don't modify!)
###

maroon = Color.fromrgb8(0x80, 0x00, 0x00)
red = Color.fromrgb8(0xff, 0x00, 0x00)
orange = Color.fromrgb8(0xff, 0xA5, 0x00)
yellow = Color.fromrgb8(0xff, 0xff, 0x00)
olive = Color.fromrgb8(0x80, 0x80, 0x00)
purple = Color.fromrgb8(0x80, 0x00, 0x80)
fuchsia = Color.fromrgb8(0xff, 0x00, 0xff)
white = Color.fromrgb8(0xff, 0xff, 0xff)
lime = Color.fromrgb8(0x00, 0xff, 0x00)
green = Color.fromrgb8(0x00, 0x80, 0x00)
navy = Color.fromrgb8(0x00, 0x00, 0x80)
blue = Color.fromrgb8(0x00, 0x00, 0xff)
aqua = Color.fromrgb8(0x00, 0xff, 0xff)
teal = Color.fromrgb8(0x00, 0x80, 0x80)
black = Color.fromrgb8(0x00, 0x00, 0x00)
silver = Color.fromrgb8(0xc0, 0xc0, 0xc0)
gray = Color.fromrgb8(0x80, 0x80, 0x80)


csscolors = {
	"maroon": maroon,
	"red": red,
	"orange": orange,
	"yellow": yellow,
	"olive": olive,
	"purple": purple,
	"fuchsia": fuchsia,
	"white": white,
	"lime": lime,
	"green": green,
	"navy": navy,
	"blue": blue,
	"aqua": aqua,
	"teal": teal,
	"black": black,
	"silver": silver,
	"gray": gray,
}


def multiply(c1, c2):
	"""
	Multiplies the colors :var:`c1` and :var:`c2`.
	"""
	c1 = c1.rgb
	c2 = c2.rgb
	return Color(c1[0]*c2[0], c1[1]*c2[1], c1[2]*c2[2])


def screen(c1, c2):
	"""
	Does a negative multiplication of the colors :var:`c1` and :var:`c2`.
	"""
	c1 = c1.rgb
	c2 = c2.rgb
	return Color(1.-(1.-c1[0])*(1.-c2[0]), 1.-(1.-c1[1])*(1.-c2[1]), 1.-(1.-c1[2])*(1.-c2[2]))


def mix(*args):
	"""
	Calculates a weighted mix of the colors from :var:`args`. Items in
	:var:`args` are either colors (in which case the weight will be 1) or
	(color, weight) tuples.
	"""
	channels = [0., 0., 0.]
	sum = 0.
	for arg in args:
		if isinstance(arg, Color):
			(c, w) = (arg, 1.)
		else:
			(c, w) = arg
		for j in xrange(3):
			channels[j] += w*c[j]
		sum += w
	if not sum:
		raise ValueError("at least one of the weights must be >0")
	for j in xrange(3):
		channels[j] /= sum
	return Color.fromrgb(*channels)
